@@ -32,6 +32,36 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <gst/gst.h>
 
+#include <string.h>
+
+gchar *
+gst_gnome_vfs_location_to_uri_string (const gchar * location)
+{
+  gchar *newloc, *ret;
+
+  if (location == NULL)
+    return NULL;
+
+  /* already an URI string? */
+  if (strstr (location, "://"))
+    return g_strdup (location);
+
+  newloc = gnome_vfs_escape_path_string (location);
+
+  if (newloc && *newloc == '/') {
+    ret = g_strdup_printf ("file://%s", newloc);
+  } else {
+    gchar *curdir;
+
+    curdir = g_get_current_dir ();
+    ret = g_strdup_printf ("file://%s/%s", curdir, newloc);
+    g_free (curdir);
+  }
+
+  g_free (newloc);
+  return ret;
+}
+
 GType
 gst_gnome_vfs_uri_get_type (void)
 {
@@ -97,6 +127,7 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE, LOCALEDIR);
  */
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif /* ENABLE_NLS */
 
   return TRUE;
