@@ -21,19 +21,15 @@
 
 /**
  * SECTION:element-oggdemux
- * @short_description: a demuxer for ogg files
+ * @see_also: <link linkend="gst-plugins-base-plugins-oggmux">oggmux</link>
+ *
+ * This element demuxes ogg files into their encoded audio and video components.
  *
  * <refsect2>
- * <para>
- * This element demuxes ogg files into their encoded audio and video components.
- * </para>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=test.ogg ! oggdemux ! vorbisdec ! audioconvert ! alsasink
- * </programlisting>
- * Decodes the vorbis audio stored inside an ogg container.
- * </para>
+ * ]| Decodes the vorbis audio stored inside an ogg container.
  * </refsect2>
  *
  * Last reviewed on 2006-12-30 (0.10.5)
@@ -61,6 +57,9 @@ GST_ELEMENT_DETAILS ("Ogg demuxer",
 
 #define GST_FLOW_LIMIT GST_FLOW_CUSTOM_ERROR
 
+#define GST_CHAIN_LOCK(ogg)     g_mutex_lock((ogg)->chain_lock)
+#define GST_CHAIN_UNLOCK(ogg)   g_mutex_unlock((ogg)->chain_lock)
+
 GST_DEBUG_CATEGORY_STATIC (gst_ogg_demux_debug);
 GST_DEBUG_CATEGORY_STATIC (gst_ogg_demux_setup_debug);
 #define GST_CAT_DEFAULT gst_ogg_demux_debug
@@ -431,6 +430,8 @@ gst_ogg_pad_reset (GstOggPad * pad)
 {
   ogg_stream_reset (&pad->stream);
 
+  GST_DEBUG_OBJECT (pad, "doing reset");
+
   /* clear continued pages */
   g_list_foreach (pad->continued, (GFunc) gst_ogg_page_free, NULL);
   g_list_free (pad->continued);
@@ -544,13 +545,15 @@ gst_ogg_pad_parse_skeleton_fishead (GstOggPad * pad, ogg_packet * packet)
   data += 8;
 
   ogg->basetime = gst_util_uint64_scale (GST_SECOND, basetime_n, basetime_d);
+  ogg->prestime = gst_util_uint64_scale (GST_SECOND, prestime_n, prestime_d);
   ogg->have_fishead = TRUE;
   pad->is_skeleton = TRUE;
   pad->start_time = GST_CLOCK_TIME_NONE;
   pad->first_granule = -1;
   pad->first_time = GST_CLOCK_TIME_NONE;
   GST_INFO_OBJECT (ogg, "skeleton fishead parsed (basetime: %"
-      GST_TIME_FORMAT ")", GST_TIME_ARGS (ogg->basetime));
+      GST_TIME_FORMAT ", prestime: %" GST_TIME_FORMAT ")",
+      GST_TIME_ARGS (ogg->basetime), GST_TIME_ARGS (ogg->prestime));
 }
 
 /* function called when a skeleton fisbone is found. Caller ensures that
@@ -558,6 +561,7 @@ gst_ogg_pad_parse_skeleton_fishead (GstOggPad * pad, ogg_packet * packet)
 static void
 gst_ogg_pad_parse_skeleton_fisbone (GstOggPad * pad, ogg_packet * packet)
 {
+  GstOggDemux *ogg = pad->ogg;
   GstOggPad *fisbone_pad;
   gint64 start_granule;
   guint32 serialno;
@@ -593,9 +597,7 @@ gst_ogg_pad_parse_skeleton_fisbone (GstOggPad * pad, ogg_packet * packet)
     /* padding */
     data += 3;
 
-    fisbone_pad->start_time = gst_annodex_granule_to_time (start_granule,
-        fisbone_pad->granulerate_n, fisbone_pad->granulerate_d,
-        fisbone_pad->granuleshift);
+    fisbone_pad->start_time = ogg->prestime - ogg->basetime;
 
     GST_INFO_OBJECT (pad->ogg, "skeleton fisbone parsed "
         "(serialno: %08x start time: %" GST_TIME_FORMAT
@@ -760,7 +762,7 @@ gst_ogg_pad_typefind (GstOggPad * pad, ogg_packet * packet)
         gst_object_sink (GST_OBJECT (element));
 
         /* FIXME, it might not be named "sink" */
-        pad->elem_pad = gst_element_get_pad (element, "sink");
+        pad->elem_pad = gst_element_get_static_pad (element, "sink");
         gst_element_set_state (element, GST_STATE_PAUSED);
         template = gst_static_pad_template_get (&internaltemplate);
         pad->elem_out = gst_pad_new_from_template (template, "internal");
@@ -774,7 +776,7 @@ gst_ogg_pad_typefind (GstOggPad * pad, ogg_packet * packet)
         {
           GstPad *p;
 
-          p = gst_element_get_pad (element, "src");
+          p = gst_element_get_static_pad (element, "src");
           if (p) {
             gst_pad_link (p, pad->elem_out);
             gst_object_unref (p);
@@ -786,7 +788,7 @@ gst_ogg_pad_typefind (GstOggPad * pad, ogg_packet * packet)
         }
       }
     }
-    g_list_free (factories);
+    gst_plugin_feature_list_free (factories);
   }
   pad->element = element;
 
@@ -834,7 +836,7 @@ no_decoder:
 decoder_error:
   {
     GST_WARNING_OBJECT (ogg, "internal decoder error");
-    return GST_FLOW_ERROR;
+    return ret;
   }
 }
 
@@ -922,8 +924,14 @@ gst_ogg_demux_chain_peer (GstOggPad * pad, ogg_packet * packet)
     goto convert_failed;
 
   /* convert to stream time */
-  if ((chain = pad->chain))
-    current_time = current_time - chain->segment_start + chain->begin_time;
+  if ((chain = pad->chain)) {
+    gint64 chain_start = 0;
+
+    if (chain->segment_start != GST_CLOCK_TIME_NONE)
+      chain_start = chain->segment_start;
+
+    current_time = current_time - chain_start + chain->begin_time;
+  }
 
   /* and store as the current position */
   gst_segment_set_last_stop (&ogg->segment, GST_FORMAT_TIME, current_time);
@@ -1405,6 +1413,7 @@ gst_ogg_demux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&ogg_demux_src_template_factory));
 }
+
 static void
 gst_ogg_demux_class_init (GstOggDemuxClass * klass)
 {
@@ -1737,6 +1746,9 @@ gst_ogg_demux_deactivate_current_chain (GstOggDemux * ogg)
   for (i = 0; i < chain->streams->len; i++) {
     GstOggPad *pad = g_array_index (chain->streams, GstOggPad *, i);
 
+    if (pad->is_skeleton)
+      continue;
+
     gst_pad_push_event (GST_PAD_CAST (pad), gst_event_new_eos ());
 
     GST_DEBUG_OBJECT (ogg, "removing pad %" GST_PTR_FORMAT, pad);
@@ -1768,36 +1780,40 @@ gst_ogg_demux_activate_chain (GstOggDemux * ogg, GstOggChain * chain,
     return TRUE;
   }
 
+  /* FIXME, should not be called with NULL */
+  if (chain != NULL) {
+    GST_DEBUG_OBJECT (ogg, "activating chain %p", chain);
+
+    /* first add the pads */
+    for (i = 0; i < chain->streams->len; i++) {
+      GstOggPad *pad;
+
+      pad = g_array_index (chain->streams, GstOggPad *, i);
+
+      if (pad->is_skeleton)
+        continue;
+
+      GST_DEBUG_OBJECT (ogg, "adding pad %" GST_PTR_FORMAT, pad);
+
+      /* mark discont */
+      pad->discont = TRUE;
+      pad->last_ret = GST_FLOW_OK;
+
+      /* activate first */
+      gst_pad_set_active (GST_PAD_CAST (pad), TRUE);
+
+      gst_element_add_pad (GST_ELEMENT (ogg), GST_PAD_CAST (pad));
+    }
+  }
+
+  /* after adding the new pads, remove the old pads */
   gst_ogg_demux_deactivate_current_chain (ogg);
 
-  /* FIXME, should not be called with NULL */
-  if (chain == NULL) {
-    ogg->current_chain = chain;
-    return TRUE;
-  }
-
-  GST_DEBUG_OBJECT (ogg, "activating chain %p", chain);
-
-  /* first add the pads */
-  for (i = 0; i < chain->streams->len; i++) {
-    GstOggPad *pad;
-
-    pad = g_array_index (chain->streams, GstOggPad *, i);
-    GST_DEBUG_OBJECT (ogg, "adding pad %" GST_PTR_FORMAT, pad);
-
-    /* mark discont */
-    pad->discont = TRUE;
-    pad->last_ret = GST_FLOW_OK;
-
-    /* activate first */
-    gst_pad_set_active (GST_PAD_CAST (pad), TRUE);
-
-    gst_element_add_pad (GST_ELEMENT (ogg), GST_PAD_CAST (pad));
-  }
-
-  gst_element_no_more_pads (GST_ELEMENT (ogg));
   ogg->current_chain = chain;
 
+  /* we are finished now */
+  gst_element_no_more_pads (GST_ELEMENT (ogg));
+
   /* FIXME, must be sent from the streaming thread */
   if (event)
     gst_ogg_demux_send_event (ogg, event);
@@ -2082,10 +2098,14 @@ gst_ogg_demux_perform_seek (GstOggDemux * ogg, GstEvent * event)
     /* create the segment event to close the current segment */
     if ((chain = ogg->current_chain)) {
       GstEvent *newseg;
+      gint64 chain_start = 0;
+
+      if (chain->segment_start != GST_CLOCK_TIME_NONE)
+        chain_start = chain->segment_start;
 
       newseg = gst_event_new_new_segment (TRUE, ogg->segment.rate,
-          GST_FORMAT_TIME, ogg->segment.start + chain->segment_start,
-          ogg->segment.last_stop + chain->segment_start, ogg->segment.time);
+          GST_FORMAT_TIME, ogg->segment.start + chain_start,
+          ogg->segment.last_stop + chain_start, ogg->segment.time);
 
       /* send segment on old chain, FIXME, must be sent from streaming thread. */
       gst_ogg_demux_send_event (ogg, newseg);
@@ -2357,7 +2377,10 @@ gst_ogg_demux_read_chain (GstOggDemux * ogg, GstOggChain ** res_chain)
   }
 
   if (ret != GST_FLOW_OK || chain == NULL) {
-    if (ret != GST_FLOW_UNEXPECTED) {
+    if (ret == GST_FLOW_OK) {
+      GST_WARNING_OBJECT (ogg, "no chain was found");
+      ret = GST_FLOW_ERROR;
+    } else if (ret != GST_FLOW_UNEXPECTED) {
       GST_WARNING_OBJECT (ogg, "failed to read chain");
     } else {
       GST_DEBUG_OBJECT (ogg, "done reading chains");
@@ -2750,19 +2773,23 @@ gst_ogg_demux_handle_page (GstOggDemux * ogg, ogg_page * page)
     chain = gst_ogg_demux_find_chain (ogg, serialno);
     if (chain) {
       GstEvent *event;
+      gint64 start = 0;
+
+      if (chain->segment_start != GST_CLOCK_TIME_NONE)
+        start = chain->segment_start;
 
       /* create the newsegment event we are going to send out */
       event = gst_event_new_new_segment (FALSE, ogg->segment.rate,
-          GST_FORMAT_TIME, chain->segment_start, chain->segment_stop,
-          chain->begin_time);
+          GST_FORMAT_TIME, start, chain->segment_stop, chain->begin_time);
 
       GST_DEBUG_OBJECT (ogg,
           "segment: start %" GST_TIME_FORMAT ", stop %" GST_TIME_FORMAT
-          ", time %" GST_TIME_FORMAT, GST_TIME_ARGS (chain->segment_start),
+          ", time %" GST_TIME_FORMAT, GST_TIME_ARGS (start),
           GST_TIME_ARGS (chain->segment_stop),
           GST_TIME_ARGS (chain->begin_time));
 
-      /* activate it as it means we have a non-header */
+      /* activate it as it means we have a non-header, this will also deactivate
+       * the currently running chain. */
       gst_ogg_demux_activate_chain (ogg, chain, event);
       pad = gst_ogg_demux_find_pad (ogg, serialno);
     } else {
@@ -2777,10 +2804,6 @@ gst_ogg_demux_handle_page (GstOggDemux * ogg, ogg_page * page)
       current_chain = ogg->current_chain;
       current_time = ogg->segment.last_stop;
 
-      if (current_chain) {
-        /* remove existing pads */
-        gst_ogg_demux_deactivate_current_chain (ogg);
-      }
       /* time of new chain is current time */
       chain_time = current_time;
 
@@ -3263,6 +3286,7 @@ gst_ogg_demux_plugin_init (GstPlugin * plugin)
   GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
       LOCALEDIR);
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
 
   return gst_element_register (plugin, "oggdemux", GST_RANK_PRIMARY,
