@@ -21,20 +21,16 @@
  * SECTION:element-theoradec
  * @see_also: theoraenc, oggdemux
  *
- * <refsect2>
- * <para>
  * This element decodes theora streams into raw video
  * <ulink url="http://www.theora.org/">Theora</ulink> is a royalty-free
  * video codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
  * Foundation</ulink>, based on the VP3 codec.
- * </para>
- * <para>
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipeline</title>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=videotestsrc.ogg ! oggdemux ! theoradec ! xvimagesink
- * </programlisting>
- * This example pipeline will decode an ogg stream and decodes the theora video. Refer to
+ * ]| This example pipeline will decode an ogg stream and decodes the theora video. Refer to
  * the theoraenc example to create the ogg file.
  * </refsect2>
  *
@@ -136,7 +132,7 @@ gst_theora_dec_class_init (GstTheoraDecClass * klass)
   g_object_class_install_property (gobject_class, ARG_CROP,
       g_param_spec_boolean ("crop", "Crop",
           "Crop the image to the visible region", THEORA_DEF_CROP,
-          (GParamFlags) G_PARAM_READWRITE));
+          (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = theora_dec_change_state;
 
@@ -789,10 +785,13 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
 
   /* calculate par
    * the info.aspect_* values reflect PAR;
-   * 0:0 is allowed and can be interpreted as 1:1, so correct for it */
+   * 0:0 is allowed and can be interpreted as 1:1, so correct for it.
+   * x:0 for other x isn't technically allowed, but it's seen in the wild and
+   * is reasonable to treat the same. 
+   */
   par_num = dec->info.aspect_numerator;
   par_den = dec->info.aspect_denominator;
-  if (par_num == 0 && par_den == 0) {
+  if (par_den == 0) {
     par_num = par_den = 1;
   }
   /* theora has:
@@ -866,8 +865,6 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
         dec->segment.format, dec->segment.start, dec->segment.stop,
         dec->segment.time);
     eret = gst_pad_push_event (dec->srcpad, event);
-    if (!eret)
-      ret = GST_FLOW_ERROR;
     dec->sent_newsegment = TRUE;
   }
 
@@ -1451,7 +1448,7 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
   dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
 
   /* peel of DISCONT flag */
-  discont = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT);
+  discont = GST_BUFFER_IS_DISCONT (buf);
 
   /* resync on DISCONT */
   if (G_UNLIKELY (discont)) {
