@@ -33,10 +33,10 @@ G_BEGIN_DECLS
 #define GST_VORBIS_PARSE(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VORBIS_PARSE,GstVorbisParse))
 #define GST_VORBIS_PARSE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VORBIS_PARSE,GstVorbisParse))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VORBIS_PARSE,GstVorbisParseClass))
 #define GST_IS_VORBIS_PARSE(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VORBIS_PARSE))
-#define GST_IS_VORBIS_PARSE_CLASS(obj) \
+#define GST_IS_VORBIS_PARSE_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VORBIS_PARSE))
 
 typedef struct _GstVorbisParse GstVorbisParse;
@@ -57,6 +57,7 @@ struct _GstVorbisParse {
   gboolean              streamheader_sent;
   GList *               streamheader;
 
+  GQueue *		event_queue;
   GQueue *		buffer_queue;
 
   vorbis_info		vi;
@@ -69,6 +70,9 @@ struct _GstVorbisParse {
 
 struct _GstVorbisParseClass {
   GstElementClass parent_class;
+
+  /* virtual functions */
+  GstFlowReturn  (*parse_packet) (GstVorbisParse * parse, GstBuffer * buf);
 };
 
 GType gst_vorbis_parse_get_type(void);
