@@ -207,7 +207,7 @@ audioringbuffer_thread_func (GstRingBuffer * buf)
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
   csink = GST_AUDIO_SINK_GET_CLASS (sink);
 
-  GST_DEBUG ("enter thread");
+  GST_DEBUG_OBJECT (sink, "enter thread");
 
   writefunc = csink->write;
   if (writefunc == NULL)
@@ -224,10 +224,11 @@ audioringbuffer_thread_func (GstRingBuffer * buf)
       left = len;
       do {
         written = writefunc (sink, readptr + written, left);
-        GST_LOG ("transfered %d bytes of %d from segment %d", written, left,
-            readseg);
+        GST_LOG_OBJECT (sink, "transfered %d bytes of %d from segment %d",
+            written, left, readseg);
         if (written < 0 || written > left) {
-          GST_WARNING ("error writing data (reason: %s), skipping segment",
+          GST_WARNING_OBJECT (sink,
+              "error writing data (reason: %s), skipping segment",
               g_strerror (errno));
           break;
         }
@@ -243,31 +244,31 @@ audioringbuffer_thread_func (GstRingBuffer * buf)
       GST_OBJECT_LOCK (abuf);
       if (!abuf->running)
         goto stop_running;
-      GST_DEBUG ("signal wait");
+      GST_DEBUG_OBJECT (sink, "signal wait");
       GST_AUDIORING_BUFFER_SIGNAL (buf);
-      GST_DEBUG ("wait for action");
+      GST_DEBUG_OBJECT (sink, "wait for action");
       GST_AUDIORING_BUFFER_WAIT (buf);
-      GST_DEBUG ("got signal");
+      GST_DEBUG_OBJECT (sink, "got signal");
       if (!abuf->running)
         goto stop_running;
-      GST_DEBUG ("continue running");
+      GST_DEBUG_OBJECT (sink, "continue running");
       GST_OBJECT_UNLOCK (abuf);
     }
   }
-  GST_DEBUG ("exit thread");
 
+  /* Will never be reached */
   return;
 
   /* ERROR */
 no_function:
   {
-    GST_DEBUG ("no write function, exit thread");
+    GST_DEBUG_OBJECT (sink, "no write function, exit thread");
     return;
   }
 stop_running:
   {
     GST_OBJECT_UNLOCK (abuf);
-    GST_DEBUG ("stop running, exit thread");
+    GST_DEBUG_OBJECT (sink, "stop running, exit thread");
     return;
   }
 }
@@ -318,7 +319,7 @@ gst_audioringbuffer_open_device (GstRingBuffer * buf)
 
 could_not_open:
   {
-    GST_DEBUG ("could not open device");
+    GST_DEBUG_OBJECT (sink, "could not open device");
     return FALSE;
   }
 }
@@ -343,7 +344,7 @@ gst_audioringbuffer_close_device (GstRingBuffer * buf)
 
 could_not_close:
   {
-    GST_DEBUG ("could not close device");
+    GST_DEBUG_OBJECT (sink, "could not close device");
     return FALSE;
   }
 }
@@ -365,8 +366,8 @@ gst_audioringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
   if (!result)
     goto could_not_prepare;
 
-  /* allocate one more segment as we need some headroom */
-  spec->segtotal++;
+  /* set latency to one more segment as we need some headroom */
+  spec->seglatency = spec->segtotal + 1;
 
   buf->data = gst_buffer_new_and_alloc (spec->segtotal * spec->segsize);
   memset (GST_BUFFER_DATA (buf->data), 0, GST_BUFFER_SIZE (buf->data));
@@ -383,7 +384,7 @@ gst_audioringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
 
 could_not_prepare:
   {
-    GST_DEBUG ("could not prepare device");
+    GST_DEBUG_OBJECT (sink, "could not prepare device");
     return FALSE;
   }
 }
@@ -402,7 +403,7 @@ gst_audioringbuffer_release (GstRingBuffer * buf)
   abuf = GST_AUDIORING_BUFFER_CAST (buf);
 
   abuf->running = FALSE;
-  GST_DEBUG ("signal wait");
+  GST_DEBUG_OBJECT (sink, "signal wait");
   GST_AUDIORING_BUFFER_SIGNAL (buf);
   GST_OBJECT_UNLOCK (buf);
 
@@ -421,11 +422,13 @@ gst_audioringbuffer_release (GstRingBuffer * buf)
   if (!result)
     goto could_not_unprepare;
 
+  GST_DEBUG_OBJECT (sink, "unprepared");
+
   return result;
 
 could_not_unprepare:
   {
-    GST_DEBUG ("could not unprepare device");
+    GST_DEBUG_OBJECT (sink, "could not unprepare device");
     return FALSE;
   }
 }
@@ -437,7 +440,7 @@ gst_audioringbuffer_start (GstRingBuffer * buf)
 
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
 
-  GST_DEBUG ("start, sending signal");
+  GST_DEBUG_OBJECT (sink, "start, sending signal");
   GST_AUDIORING_BUFFER_SIGNAL (buf);
 
   return TRUE;
@@ -454,9 +457,9 @@ gst_audioringbuffer_pause (GstRingBuffer * buf)
 
   /* unblock any pending writes to the audio device */
   if (csink->reset) {
-    GST_DEBUG ("reset...");
+    GST_DEBUG_OBJECT (sink, "reset...");
     csink->reset (sink);
-    GST_DEBUG ("reset done");
+    GST_DEBUG_OBJECT (sink, "reset done");
   }
 
   return TRUE;
@@ -475,16 +478,17 @@ gst_audioringbuffer_stop (GstRingBuffer * buf)
 
   /* unblock any pending writes to the audio device */
   if (csink->reset) {
-    GST_DEBUG ("reset...");
+    GST_DEBUG_OBJECT (sink, "reset...");
     csink->reset (sink);
-    GST_DEBUG ("reset done");
+    GST_DEBUG_OBJECT (sink, "reset done");
   }
-
+#if 0
   if (abuf->running) {
-    GST_DEBUG ("stop, waiting...");
+    GST_DEBUG_OBJECT (sink, "stop, waiting...");
     GST_AUDIORING_BUFFER_WAIT (buf);
-    GST_DEBUG ("stopped");
+    GST_DEBUG_OBJECT (sink, "stopped");
   }
+#endif
 
   return TRUE;
 }
@@ -558,9 +562,9 @@ gst_audio_sink_create_ringbuffer (GstBaseAudioSink * sink)
 {
   GstRingBuffer *buffer;
 
-  GST_DEBUG ("creating ringbuffer");
+  GST_DEBUG_OBJECT (sink, "creating ringbuffer");
   buffer = g_object_new (GST_TYPE_AUDIORING_BUFFER, NULL);
-  GST_DEBUG ("created ringbuffer @%p", buffer);
+  GST_DEBUG_OBJECT (sink, "created ringbuffer @%p", buffer);
 
   return buffer;
 }
