@@ -19,7 +19,6 @@
 
 /**
  * SECTION:element-decodebin2
- * @short_description: Next-generation automatic decoding bin
  *
  * #GstBin that auto-magically constructs a decoding pipeline using available
  * decoders and demuxers via auto-plugging.
@@ -1836,6 +1835,14 @@ gst_decode_group_check_if_drained (GstDecodeGroup * group)
 
   GST_LOG ("group : %p", group);
 
+  DECODE_BIN_LOCK (dbin);
+
+  /* Ensure we only emit the drained signal once, for this group */
+  if (group->drained) {
+    drained = FALSE;
+    goto done;
+  }
+
   for (tmp = group->endpads; tmp; tmp = g_list_next (tmp)) {
     GstDecodePad *dpad = (GstDecodePad *) tmp->data;
 
@@ -1849,10 +1856,9 @@ gst_decode_group_check_if_drained (GstDecodeGroup * group)
 
   group->drained = drained;
   if (!drained)
-    return;
+    goto done;
 
   /* we are drained. Check if there is a next group to activate */
-  DECODE_BIN_LOCK (dbin);
   if ((group == dbin->activegroup) && dbin->groups) {
     GST_DEBUG_OBJECT (dbin, "Switching to new group");
 
@@ -1863,6 +1869,8 @@ gst_decode_group_check_if_drained (GstDecodeGroup * group)
     /* we're not yet drained now */
     drained = FALSE;
   }
+
+done:
   DECODE_BIN_UNLOCK (dbin);
 
   if (drained) {
@@ -2412,6 +2420,7 @@ gst_decode_bin_plugin_init (GstPlugin * plugin)
   GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
       LOCALEDIR);
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif /* ENABLE_NLS */
 
   return gst_element_register (plugin, "decodebin2", GST_RANK_NONE,
