@@ -50,7 +50,9 @@ typedef enum
   GST_SUB_PARSE_FORMAT_SUBRIP = 2,
   GST_SUB_PARSE_FORMAT_MPSUB = 3,
   GST_SUB_PARSE_FORMAT_SAMI = 4,
-  GST_SUB_PARSE_FORMAT_TMPLAYER = 5
+  GST_SUB_PARSE_FORMAT_TMPLAYER = 5,
+  GST_SUB_PARSE_FORMAT_MPL2 = 6,
+  GST_SUB_PARSE_FORMAT_SUBVIEWER = 7
 } GstSubParseFormat;
 
 typedef struct {
@@ -58,8 +60,10 @@ typedef struct {
   GString *buf;
   guint64  start_time;
   guint64  duration;
+  guint64  max_duration; /* to clamp duration, 0 = no limit (used by tmplayer parser) */
   GstSegment *segment;
   gpointer user_data;
+  gdouble  fps;          /* used by microdvd parser */
 } ParserState;
 
 typedef gchar* (*Parser) (ParserState *state, const gchar *line);
@@ -82,9 +86,9 @@ struct _GstSubParse {
   guint64 next_offset;
   
   /* Segment */
-  GstSegment *segment;
+  GstSegment    segment;
   GstSeekFlags  segment_flags;
-  gboolean need_segment;
+  gboolean      need_segment;
   
   gboolean flushing;
   gboolean valid_utf8;
