@@ -20,14 +20,13 @@
 /**
  * SECTION:element-ximagesink
  *
- * <refsect2>
- * <para>
  * XImageSink renders video frames to a drawable (XWindow) on a local or remote
  * display. This element can receive a Window ID from the application through
  * the XOverlay interface and will then render video frames in this drawable.
  * If no Window ID was provided by the application, the element will create its
  * own internal window and render into it.
- * </para>
+ *
+ * <refsect2>
  * <title>Scaling</title>
  * <para>
  * As standard XImage rendering to a drawable is not scaled, XImageSink will use
@@ -37,12 +36,13 @@
  * or that an element generating the video frames can generate them with a 
  * different geometry. This mechanism is handled during buffer allocations, for
  * each allocation request the video sink will check the drawable geometry, look
- * at the
- * <link linkend="GstXImageSink--force-aspect-ratio">force-aspect-ratio</link>
- * property, calculate the geometry of desired video frames and then check that
- * the peer pad accept those new caps. If it does it will then allocate a buffer
- * in video memory with this new geometry and return it with the new caps.
+ * at the #GstXImageSink:force-aspect-ratio property, calculate the geometry of
+ * desired video frames and then check that the peer pad accept those new caps.
+ * If it does it will then allocate a buffer in video memory with this new
+ * geometry and return it with the new caps.
  * </para>
+ * </refsect2>
+ * <refsect2>
  * <title>Events</title>
  * <para>
  * XImageSink creates a thread to handle events coming from the drawable. There
@@ -55,11 +55,12 @@
  * paused, it will receive expose events from the drawable and draw the latest
  * frame with correct borders/aspect-ratio.
  * </para>
+ * </refsect2>
+ * <refsect2>
  * <title>Pixel aspect ratio</title>
  * <para>
  * When changing state to GST_STATE_READY, XImageSink will open a connection to
- * the display specified in the
- * <link linkend="GstXImageSink--display">display</link> property or the default
+ * the display specified in the #GstXImageSink:display property or the default
  * display if nothing specified. Once this connection is open it will inspect 
  * the display configuration including the physical display geometry and 
  * then calculate the pixel aspect ratio. When caps negotiation will occur, the
@@ -67,43 +68,34 @@
  * sure that incoming video frames will have the correct pixel aspect ratio for
  * this display. Sometimes the calculated pixel aspect ratio can be wrong, it is
  * then possible to enforce a specific pixel aspect ratio using the
- * <link linkend="GstXImageSink--pixel-aspect-ratio">pixel-aspect-ratio</link>
- * property.
+ * #GstXImageSink:pixel-aspect-ratio property.
  * </para>
+ * </refsect2>
+ * <refsect2>
  * <title>Examples</title>
- * <para>
- * Here is a simple pipeline to test reverse negotiation :
- * <programlisting>
+ * |[
  * gst-launch -v videotestsrc ! queue ! ximagesink
- * </programlisting>
- * When the test video signal appears you can resize the window and see that
- * scaled buffers of the desired size are going to arrive with a short delay.
- * This illustrates how buffers of desired size are allocated along the way.
- * If you take away the queue, scaling will happen almost immediately.
- * </para>
- * <para>
- * Here is a simple pipeline to test navigation events :
- * <programlisting>
+ * ]| A pipeline to test reverse negotiation. When the test video signal appears
+ * you can resize the window and see that scaled buffers of the desired size are
+ * going to arrive with a short delay. This illustrates how buffers of desired
+ * size are allocated along the way. If you take away the queue, scaling will
+ * happen almost immediately.
+ * |[
  * gst-launch -v videotestsrc ! navigationtest ! ffmpegcolorspace ! ximagesink
- * </programlisting>
+ * ]| A pipeline to test navigation events.
  * While moving the mouse pointer over the test signal you will see a black box
  * following the mouse pointer. If you press the mouse button somewhere on the 
  * video and release it somewhere else a green box will appear where you pressed
  * the button and a red one where you released it. (The navigationtest element
  * is part of gst-plugins-good.)
- * </para>
- * <para>
- * Here is a simple pipeline to test pixel aspect ratio :
- * <programlisting>
+ * |[
  * gst-launch -v videotestsrc ! video/x-raw-rgb, pixel-aspect-ratio=(fraction)4/3 ! videoscale ! ximagesink
- * </programlisting>
- * This is faking a 4/3 pixel aspect ratio caps on video frames produced by
+ * ]| This is faking a 4/3 pixel aspect ratio caps on video frames produced by
  * videotestsrc, in most cases the pixel aspect ratio of the display will be
  * 1/1. This means that videoscale will have to do the scaling to convert 
  * incoming frames to a size that will match the display pixel aspect ratio
  * (from 320x240 to 320x180 in this case). Note that you might have to escape 
  * some characters for your shell like '\(fraction\)'.
- * </para>
  * </refsect2>
  */
 
@@ -369,22 +361,24 @@ gst_ximagesink_check_xshm_calls (GstXImageSink * ximagesink,
     goto beach;
   }
 
-  /* Delete the shared memory segment as soon as we manage to attach. 
-   * This way, it will be deleted as soon as we detach later, and not
-   * leaked if we crash. */
-  shmctl (SHMInfo.shmid, IPC_RMID, NULL);
-
   ximage->data = SHMInfo.shmaddr;
   SHMInfo.readOnly = FALSE;
 
   if (XShmAttach (xcontext->disp, &SHMInfo) == 0) {
     GST_WARNING ("Failed to XShmAttach");
+    /* Clean up shm seg */
+    shmctl (SHMInfo.shmid, IPC_RMID, NULL);
     goto beach;
   }
 
   /* Sync to ensure we see any errors we caused */
   XSync (xcontext->disp, FALSE);
 
+  /* Delete the shared memory segment as soon as everyone is attached. 
+   * This way, it will be deleted as soon as we detach later, and not
+   * leaked if we crash. */
+  shmctl (SHMInfo.shmid, IPC_RMID, NULL);
+
   if (!error_caught) {
     did_attach = TRUE;
     /* store whether we succeeded in result */
@@ -488,15 +482,13 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
       goto beach;
     }
 
-    /* Now that we've attached, we can delete the shared memory segment.
-     * This way, it will be deleted as soon as we detach later, and not
-     * leaked if we crash. */
-    shmctl (ximage->SHMInfo.shmid, IPC_RMID, NULL);
-
     ximage->ximage->data = ximage->SHMInfo.shmaddr;
     ximage->SHMInfo.readOnly = FALSE;
 
     if (XShmAttach (ximagesink->xcontext->disp, &ximage->SHMInfo) == 0) {
+      /* Clean up shm seg */
+      shmctl (ximage->SHMInfo.shmid, IPC_RMID, NULL);
+
       g_mutex_unlock (ximagesink->x_lock);
       GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE,
           ("Failed to create output image buffer of %dx%d pixels",
@@ -505,6 +497,12 @@ gst_ximagesink_ximage_new (GstXImageSink * ximagesink, GstCaps * caps)
     }
 
     XSync (ximagesink->xcontext->disp, FALSE);
+
+    /* Now that everyone has attached, we can delete the shared memory segment.
+     * This way, it will be deleted as soon as we detach later, and not
+     * leaked if we crash. */
+    shmctl (ximage->SHMInfo.shmid, IPC_RMID, NULL);
+
   } else
 #endif /* HAVE_XSHM */
   {
@@ -694,8 +692,8 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
   }
 
   /* Draw borders when displaying the first frame. After this
-     draw borders only on expose event. */
-  if (!ximagesink->cur_image) {
+     draw borders only on expose event or caps change (ximagesink->draw_border = TRUE). */
+  if (!ximagesink->cur_image || ximagesink->draw_border) {
     draw_border = TRUE;
   }
 
@@ -735,6 +733,7 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
   if (draw_border) {
     gst_ximagesink_xwindow_draw_borders (ximagesink, ximagesink->xwindow,
         result);
+    ximagesink->draw_border = FALSE;
   }
 #ifdef HAVE_XSHM
   if (ximagesink->xcontext->use_xshm) {
@@ -1465,6 +1464,8 @@ gst_ximagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
     ximagesink->xwindow = gst_ximagesink_xwindow_new (ximagesink,
         GST_VIDEO_SINK_WIDTH (ximagesink), GST_VIDEO_SINK_HEIGHT (ximagesink));
   }
+  /* Remember to draw borders for next frame */
+  ximagesink->draw_border = TRUE;
   g_mutex_unlock (ximagesink->flow_lock);
 
   /* If our ximage has changed we destroy it, next chain iteration will create
