@@ -24,83 +24,67 @@
 #endif
 
 #include <stdlib.h>
+#include <glib.h>
+#include <gtk/gtk.h>
+#include <gst/gst.h>
 #include <string.h>
 
-#include <glib.h>
+#include <X11/Xlib.h>
 #include <gdk/gdkx.h>
-#include <gtk/gtk.h>
-
-#include <gst/gst.h>
 #include <gst/interfaces/xoverlay.h>
-#include <gst/interfaces/propertyprobe.h>
 
 static GtkWidget *video_window = NULL;
 static GstElement *sink = NULL;
-static gulong embed_xid = 0;
-static GdkColor trans_color;
-static gboolean trans_color_set = FALSE;
+static guint embed_xid = 0;
+static GdkGC *trans_gc = NULL;
 
-static void
-redraw_overlay (GtkWidget * widget)
+static GstBusSyncReply
+bus_sync_handler (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
 {
-  GtkAllocation allocation;
-  GdkWindow *window = gtk_widget_get_window (widget);
-  cairo_t *cr;
+  if ((GST_MESSAGE_TYPE (message) == GST_MESSAGE_ELEMENT) &&
+      gst_structure_has_name (message->structure, "prepare-xwindow-id")) {
+    GstElement *element = GST_ELEMENT (GST_MESSAGE_SRC (message));
 
-  cr = gdk_cairo_create (window);
-  gtk_widget_get_allocation (widget, &allocation);
-
-  cairo_set_source_rgb (cr, 1, 1, 1);
-  cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
-  cairo_fill (cr);
-
-  if (trans_color_set) {
-    guint x, y;
-    guint h = allocation.height * 0.75;
-
-    gdk_cairo_set_source_color (cr, &trans_color);
-    cairo_rectangle (cr, 0, 0, allocation.width, h);
-    cairo_fill (cr);
-
-    for (y = h; y < allocation.height; y++) {
-      for (x = 0; x < allocation.width; x++) {
-        if (((x & 1) || (y & 1)) && (x & 1) != (y & 1)) {
-          cairo_move_to (cr, x, y);
-          cairo_paint (cr);
-        }
-      }
+    g_print ("got prepare-xwindow-id\n");
+    if (!embed_xid) {
+      embed_xid = GDK_WINDOW_XID (GDK_WINDOW (video_window->window));
     }
+
+    if (g_object_class_find_property (G_OBJECT_GET_CLASS (element),
+            "force-aspect-ratio")) {
+      g_object_set (element, "force-aspect-ratio", TRUE, NULL);
+    }
+
+    gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (GST_MESSAGE_SRC (message)),
+        embed_xid);
   }
+  return GST_BUS_PASS;
 }
 
 static gboolean
 handle_resize_cb (GtkWidget * widget, GdkEventConfigure * event, gpointer data)
 {
-  redraw_overlay (widget);
+  gdk_draw_rectangle (widget->window, widget->style->white_gc, TRUE,
+      0, 0, widget->allocation.width, widget->allocation.height);
+
+  if (trans_gc) {
+    guint x, y;
+    guint h = widget->allocation.height * 0.75;
+
+    gdk_draw_rectangle (widget->window, trans_gc, TRUE,
+        0, 0, widget->allocation.width, h);
+
+    for (y = h; y < widget->allocation.height; y++) {
+      for (x = 0; x < widget->allocation.width; x++) {
+        if (((x & 1) || (y & 1)) && (x & 1) != (y & 1)) {
+          gdk_draw_point (widget->window, trans_gc, x, y);
+        }
+      }
+    }
+  }
   return FALSE;
 }
 
-static gboolean
-draw_cb (GtkWidget * widget, cairo_t * cr, gpointer data)
-{
-  redraw_overlay (widget);
-  return FALSE;
-}
-
-static void
-realize_cb (GtkWidget * widget, gpointer data)
-{
-  GdkWindow *window = gtk_widget_get_window (widget);
-
-  /* This is here just for pedagogical purposes, GDK_WINDOW_XID will call it
-   * as well */
-  if (!gdk_window_ensure_native (window))
-    g_error ("Couldn't create native window needed for GstXOverlay!");
-
-  embed_xid = GDK_WINDOW_XID (window);
-  g_print ("Window realize: video window XID = %lu\n", embed_xid);
-}
-
 static void
 msg_state_changed (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
 {
@@ -117,19 +101,20 @@ msg_state_changed (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
 
     /* When state of the pipeline changes to paused or playing we start updating scale */
     switch (GST_STATE_TRANSITION (old, new)) {
-      case GST_STATE_CHANGE_READY_TO_PAUSED:{
+      case GST_STATE_CHANGE_READY_TO_PAUSED:
         g_object_get (G_OBJECT (sink), "colorkey", &color, NULL);
         if (color != -1) {
-          trans_color.red = (color & 0xff0000) >> 8;
-          trans_color.green = (color & 0xff00);
-          trans_color.blue = (color & 0xff) << 8;
-          trans_color_set = TRUE;
-        } else {
-          trans_color_set = FALSE;
+          GdkColor trans_color = { 0,
+            (color & 0xff0000) >> 8,
+            (color & 0xff00),
+            (color & 0xff) << 8
+          };
+
+          trans_gc = gdk_gc_new (video_window->window);
+          gdk_gc_set_rgb_fg_color (trans_gc, &trans_color);
         }
         handle_resize_cb (video_window, NULL, NULL);
         break;
-      }
       default:
         break;
     }
@@ -142,40 +127,25 @@ window_closed (GtkWidget * widget, GdkEvent * event, gpointer user_data)
   GstElement *pipeline = user_data;
 
   g_print ("stopping\n");
-  gtk_widget_hide (widget);
+  gtk_widget_hide_all (widget);
   gst_element_set_state (pipeline, GST_STATE_NULL);
   gtk_main_quit ();
 }
 
-static gboolean
-start_pipeline (gpointer user_data)
-{
-  GstElement *pipeline = GST_ELEMENT (user_data);
-  GstStateChangeReturn sret;
-
-  sret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
-  if (sret == GST_STATE_CHANGE_FAILURE) {
-    gst_element_set_state (pipeline, GST_STATE_NULL);
-    gst_object_unref (pipeline);
-    gtk_main_quit ();
-  }
-  return FALSE;
-}
-
 int
 main (int argc, char **argv)
 {
   GtkWidget *window;
   GstElement *pipeline, *src;
   GstBus *bus;
-  GstStateChangeReturn sret;
-  GstPropertyProbe *probe;
-  GValueArray *arr;
 
-#if !GLIB_CHECK_VERSION (2, 31, 0)
   if (!g_thread_supported ())
     g_thread_init (NULL);
-#endif
+
+  if (!XInitThreads ()) {
+    g_print ("XInitThreads failed\n");
+    exit (-1);
+  }
 
   gst_init (&argc, &argv);
   gtk_init (&argc, &argv);
@@ -188,40 +158,13 @@ main (int argc, char **argv)
   gst_bin_add_many (GST_BIN (pipeline), src, sink, NULL);
   gst_element_link (src, sink);
 
-#define COLOR_GRAY 0x7F7F7F
-
-  g_object_set (G_OBJECT (sink), "autopaint-colorkey", FALSE,
-      "force-aspect-ratio", TRUE, "draw-borders", FALSE,
-      "colorkey", COLOR_GRAY, NULL);
-
-  /* check xvimagesink capabilities */
-  sret = gst_element_set_state (pipeline, GST_STATE_READY);
-  if (sret == GST_STATE_CHANGE_FAILURE) {
-    g_printerr ("Can't set pipeline to READY\n");
-    gst_object_unref (pipeline);
-    return -1;
-  }
-
-  probe = GST_PROPERTY_PROBE (sink);
-  if (!probe) {
-    g_printerr ("Can't probe sink\n");
-    gst_element_set_state (pipeline, GST_STATE_NULL);
-    gst_object_unref (pipeline);
-    return -1;
-  }
-  arr =
-      gst_property_probe_probe_and_get_values_name (probe,
-      "autopaint-colorkey");
-  if (!arr || !arr->n_values) {
-    g_printerr ("Can't disable autopaint-colorkey property\n");
-    gst_element_set_state (pipeline, GST_STATE_NULL);
-    gst_object_unref (pipeline);
-    return -1;
-  }
-  if (arr)
-    g_value_array_free (arr);
+  g_object_set (G_OBJECT (sink),
+      "autopaint-colorkey", FALSE,
+      "force-aspect-ratio", TRUE, "draw-borders", FALSE, NULL);
 
   bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
+  gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_handler,
+      pipeline);
   gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
   g_signal_connect (bus, "message::state-changed",
       G_CALLBACK (msg_state_changed), pipeline);
@@ -230,40 +173,22 @@ main (int argc, char **argv)
   /* prepare the ui */
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  video_window = gtk_drawing_area_new ();
+  gtk_widget_set_double_buffered (video_window, FALSE);
+
+  gtk_window_set_default_size (GTK_WINDOW (window), 320, 240);
+  gtk_container_add (GTK_CONTAINER (window), video_window);
   g_signal_connect (G_OBJECT (window), "delete-event",
       G_CALLBACK (window_closed), (gpointer) pipeline);
-  gtk_window_set_default_size (GTK_WINDOW (window), 320, 240);
-
-  video_window = gtk_drawing_area_new ();
   g_signal_connect (G_OBJECT (video_window), "configure-event",
       G_CALLBACK (handle_resize_cb), NULL);
-  g_signal_connect (G_OBJECT (video_window), "draw",
-      G_CALLBACK (draw_cb), NULL);
-  g_signal_connect (video_window, "realize", G_CALLBACK (realize_cb), NULL);
-  gtk_widget_set_double_buffered (video_window, FALSE);
-  gtk_container_add (GTK_CONTAINER (window), video_window);
-
-  /* show the gui and play */
 
+  /* show the gui. */
   gtk_widget_show_all (window);
 
-  /* realize window now so that the video window gets created and we can
-   * obtain its XID before the pipeline is started up and the videosink
-   * asks for the XID of the window to render onto */
-  gtk_widget_realize (window);
-
-  /* we should have the XID now */
-  g_assert (embed_xid != 0);
-
-  /* we know what the video sink is in this case (xvimagesink), so we can
-   * just set it directly here now (instead of waiting for a prepare-xwindow-id
-   * element message in a sync bus handler and setting it there) */
-  g_print ("setting XID %lu\n", embed_xid);
-  gst_x_overlay_set_window_handle (GST_X_OVERLAY (sink), embed_xid);
-
-  g_idle_add (start_pipeline, pipeline);
+  //connect_bus_signals (pipeline);
+  gst_element_set_state (pipeline, GST_STATE_PLAYING);
   gtk_main ();
-
   gst_object_unref (pipeline);
 
   return 0;
